@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import hashlib
+import sys
 import tempfile
 from pathlib import Path
 from typing import TYPE_CHECKING, Literal, cast
@@ -17,7 +18,7 @@ from airbyte._executors.local import PathExecutor
 from airbyte._executors.python import VenvExecutor
 from airbyte._util.meta import which
 from airbyte._util.telemetry import EventState, log_install_state  # Non-public API
-from airbyte.constants import AIRBYTE_OFFLINE_MODE, TEMP_DIR_OVERRIDE
+from airbyte.constants import AIRBYTE_OFFLINE_MODE, DEFAULT_PROJECT_DIR, TEMP_DIR_OVERRIDE
 from airbyte.sources.registry import ConnectorMetadata, InstallType, get_connector_metadata
 from airbyte.version import get_version
 
@@ -147,7 +148,7 @@ def _get_local_executor(
 
     # `local_executable` is now a Path object
 
-    print(f"Using local `{name}` executable: {local_executable!s}")
+    print(f"Using local `{name}` executable: {local_executable!s}", file=sys.stderr)
     return PathExecutor(
         name=name,
         path=local_executable,
@@ -165,6 +166,7 @@ def get_connector_executor(  # noqa: PLR0912, PLR0913, PLR0914, PLR0915, C901 #
     source_manifest: bool | dict | Path | str | None = None,
     install_if_missing: bool = True,
     install_root: Path | None = None,
+    use_python: bool | Path | str | None = None,
 ) -> Executor:
     """This factory function creates an executor for a connector.
 
@@ -174,11 +176,18 @@ def get_connector_executor(  # noqa: PLR0912, PLR0913, PLR0914, PLR0915, C901 #
         [
             bool(local_executable),
             bool(docker_image),
-            bool(pip_url),
+            bool(pip_url) or bool(use_python),
             bool(source_manifest),
         ]
     )
 
+    if use_python is False:
+        docker_image = True
+
+    if use_python is None and pip_url is not None:
+        # If pip_url is set, we assume the user wants to use Python.
+        use_python = True
+
     if version and pip_url:
         raise exc.PyAirbyteInputError(
             message=(
@@ -273,7 +282,7 @@ def get_connector_executor(  # noqa: PLR0912, PLR0913, PLR0914, PLR0915, C901 #
         host_temp_dir = TEMP_DIR_OVERRIDE or Path(tempfile.gettempdir())
         container_temp_dir = DEFAULT_AIRBYTE_CONTAINER_TEMP_DIR
 
-        local_mount_dir = Path().absolute() / name
+        local_mount_dir = DEFAULT_PROJECT_DIR / name
         local_mount_dir.mkdir(exist_ok=True)
 
         volumes = {
@@ -296,11 +305,20 @@ def get_connector_executor(  # noqa: PLR0912, PLR0913, PLR0914, PLR0915, C901 #
 
         return DockerExecutor(
             name=name,
+            image_name_full=docker_image,
             executable=docker_cmd,
             volumes=volumes,
         )
 
     if source_manifest:
+        if (
+            isinstance(source_manifest, str)
+            and len(source_manifest.splitlines()) == 1
+            and not source_manifest.startswith(("http://", "https://"))
+        ):
+            # If source_manifest is a single line string and not a URL, assume it's a file path
+            source_manifest = Path(source_manifest).expanduser()
+
         if isinstance(source_manifest, dict | Path):
             components_py_path: Path | None = None
             if isinstance(source_manifest, Path):
@@ -340,6 +358,7 @@ def get_connector_executor(  # noqa: PLR0912, PLR0913, PLR0914, PLR0915, C901 #
             target_version=version,
             pip_url=pip_url,
             install_root=install_root,
+            use_python=use_python,
         )
         if install_if_missing:
             executor.ensure_installation()
