@@ -14,12 +14,14 @@ directly. This will ensure a single source of truth when mapping between the `ai
 from __future__ import annotations
 
 import json
+from http import HTTPStatus
 from typing import TYPE_CHECKING, Any, Literal
 
 import airbyte_api
 import requests
 from airbyte_api import api, models
 
+from airbyte.constants import CLOUD_API_ROOT, CLOUD_CONFIG_API_ROOT
 from airbyte.exceptions import (
     AirbyteConnectionSyncError,
     AirbyteError,
@@ -44,20 +46,6 @@ if TYPE_CHECKING:
 
 JOB_WAIT_INTERVAL_SECS = 2.0
 JOB_WAIT_TIMEOUT_SECS_DEFAULT = 60 * 60  # 1 hour
-CLOUD_API_ROOT = "https://api.airbyte.com/v1"
-"""The Airbyte Cloud API root URL.
-
-This is the root URL for the Airbyte Cloud API. It is used to interact with the Airbyte Cloud API
-and is the default API root for the `CloudWorkspace` class.
-- https://reference.airbyte.com/reference/getting-started
-"""
-CLOUD_CONFIG_API_ROOT = "https://cloud.airbyte.com/api/v1"
-"""Internal-Use API Root, aka Airbyte "Config API".
-
-Documentation:
-- https://docs.airbyte.com/api-documentation#configuration-api-deprecated
-- https://github.com/airbytehq/airbyte-platform-internal/blob/master/oss/airbyte-api/server-api/src/main/openapi/config.yaml
-"""
 
 
 def status_ok(status_code: int) -> bool:
@@ -73,6 +61,19 @@ def get_config_api_root(api_root: str) -> str:
     raise NotImplementedError("Configuration API root not implemented for this API root.")
 
 
+def get_web_url_root(api_root: str) -> str:
+    """Get the web URL root from the main API root.
+
+    # TODO: This does not return a valid URL for self-managed instances, due to not knowing the
+    # web URL root. Logged here:
+    # - https://github.com/airbytehq/PyAirbyte/issues/563
+    """
+    if api_root == CLOUD_API_ROOT:
+        return "https://cloud.airbyte.com"
+
+    return api_root
+
+
 def get_airbyte_server_instance(
     *,
     api_root: str,
@@ -150,25 +151,34 @@ def list_connections(
         client_secret=client_secret,
         api_root=api_root,
     )
-    response = airbyte_instance.connections.list_connections(
-        api.ListConnectionsRequest(
-            workspace_ids=[workspace_id],
-        ),
-    )
-
-    if not status_ok(response.status_code) and response.connections_response:
-        raise AirbyteError(
-            context={
-                "workspace_id": workspace_id,
-                "response": response,
-            }
+    result: list[models.ConnectionResponse] = []
+    has_more = True
+    offset, page_size = 0, 100
+    while has_more:
+        response = airbyte_instance.connections.list_connections(
+            api.ListConnectionsRequest(
+                workspace_ids=[workspace_id],
+                offset=offset,
+                limit=page_size,
+            ),
         )
-    assert response.connections_response is not None
-    return [
-        connection
-        for connection in response.connections_response.data
-        if name_filter(connection.name)
-    ]
+        has_more = bool(response.connections_response and response.connections_response.next)
+        offset += page_size
+
+        if not status_ok(response.status_code) and response.connections_response:
+            raise AirbyteError(
+                context={
+                    "workspace_id": workspace_id,
+                    "response": response,
+                }
+            )
+        assert response.connections_response is not None
+        result += [
+            connection
+            for connection in response.connections_response.data
+            if name_filter(connection.name)
+        ]
+    return result
 
 
 def list_workspaces(
@@ -192,24 +202,32 @@ def list_workspaces(
         client_secret=client_secret,
         api_root=api_root,
     )
-
-    response: api.ListWorkspacesResponse = airbyte_instance.workspaces.list_workspaces(
-        api.ListWorkspacesRequest(
-            workspace_ids=[workspace_id],
-        ),
-    )
-
-    if not status_ok(response.status_code) and response.workspaces_response:
-        raise AirbyteError(
-            context={
-                "workspace_id": workspace_id,
-                "response": response,
-            }
+    result: list[models.WorkspaceResponse] = []
+    has_more = True
+    offset, page_size = 0, 100
+    while has_more:
+        response: api.ListWorkspacesResponse = airbyte_instance.workspaces.list_workspaces(
+            api.ListWorkspacesRequest(workspace_ids=[workspace_id], offset=offset, limit=page_size),
         )
-    assert response.workspaces_response is not None
-    return [
-        workspace for workspace in response.workspaces_response.data if name_filter(workspace.name)
-    ]
+        has_more = bool(response.workspaces_response and response.workspaces_response.next)
+        offset += page_size
+
+        if not status_ok(response.status_code) and response.workspaces_response:
+            raise AirbyteError(
+                context={
+                    "workspace_id": workspace_id,
+                    "response": response,
+                }
+            )
+
+        assert response.workspaces_response is not None
+        result += [
+            workspace
+            for workspace in response.workspaces_response.data
+            if name_filter(workspace.name)
+        ]
+
+    return result
 
 
 def list_sources(
@@ -233,21 +251,31 @@ def list_sources(
         client_secret=client_secret,
         api_root=api_root,
     )
-    response: api.ListSourcesResponse = airbyte_instance.sources.list_sources(
-        api.ListSourcesRequest(
-            workspace_ids=[workspace_id],
-        ),
-    )
-
-    if not status_ok(response.status_code) and response.sources_response:
-        raise AirbyteError(
-            context={
-                "workspace_id": workspace_id,
-                "response": response,
-            }
+    result: list[models.SourceResponse] = []
+    has_more = True
+    offset, page_size = 0, 100
+    while has_more:
+        response: api.ListSourcesResponse = airbyte_instance.sources.list_sources(
+            api.ListSourcesRequest(
+                workspace_ids=[workspace_id],
+                offset=offset,
+                limit=page_size,
+            ),
         )
-    assert response.sources_response is not None
-    return [source for source in response.sources_response.data if name_filter(source.name)]
+        has_more = bool(response.sources_response and response.sources_response.next)
+        offset += page_size
+
+        if not status_ok(response.status_code) and response.sources_response:
+            raise AirbyteError(
+                context={
+                    "workspace_id": workspace_id,
+                    "response": response,
+                }
+            )
+        assert response.sources_response is not None
+        result += [source for source in response.sources_response.data if name_filter(source.name)]
+
+    return result
 
 
 def list_destinations(
@@ -271,25 +299,35 @@ def list_destinations(
         client_secret=client_secret,
         api_root=api_root,
     )
-    response = airbyte_instance.destinations.list_destinations(
-        api.ListDestinationsRequest(
-            workspace_ids=[workspace_id],
-        ),
-    )
-
-    if not status_ok(response.status_code) and response.destinations_response:
-        raise AirbyteError(
-            context={
-                "workspace_id": workspace_id,
-                "response": response,
-            }
+    result: list[models.DestinationResponse] = []
+    has_more = True
+    offset, page_size = 0, 100
+    while has_more:
+        response = airbyte_instance.destinations.list_destinations(
+            api.ListDestinationsRequest(
+                workspace_ids=[workspace_id],
+                offset=offset,
+                limit=page_size,
+            ),
         )
-    assert response.destinations_response is not None
-    return [
-        destination
-        for destination in response.destinations_response.data
-        if name_filter(destination.name)
-    ]
+        has_more = bool(response.destinations_response and response.destinations_response.next)
+        offset += page_size
+
+        if not status_ok(response.status_code) and response.destinations_response:
+            raise AirbyteError(
+                context={
+                    "workspace_id": workspace_id,
+                    "response": response,
+                }
+            )
+        assert response.destinations_response is not None
+        result += [
+            destination
+            for destination in response.destinations_response.data
+            if name_filter(destination.name)
+        ]
+
+    return result
 
 
 # Get and run connections
@@ -369,7 +407,7 @@ def run_connection(
 def get_job_logs(
     workspace_id: str,
     connection_id: str,
-    limit: int = 20,
+    limit: int = 100,
     *,
     api_root: str,
     client_id: SecretString,
@@ -522,6 +560,28 @@ def delete_source(
         )
 
 
+# Utility function
+
+
+def _get_destination_type_str(
+    destination: DestinationConfiguration | dict[str, Any],
+) -> str:
+    if isinstance(destination, dict):
+        destination_type = destination.get("destinationType")
+    else:
+        destination_type = getattr(destination, "DESTINATION_TYPE", None)
+
+    if not destination_type or not isinstance(destination_type, str):
+        raise PyAirbyteInputError(
+            message="Could not determine destination type from configuration.",
+            context={
+                "destination": destination,
+            },
+        )
+
+    return destination_type
+
+
 # Create, get, and delete destinations
 
 
@@ -540,8 +600,14 @@ def create_destination(
         client_secret=client_secret,
         api_root=api_root,
     )
+    definition_id_override: str | None = None
+    if _get_destination_type_str(config) == "dev-null":
+        # TODO: We have to hard-code the definition ID for dev-null destination. (important-comment)
+        #  https://github.com/airbytehq/PyAirbyte/issues/743
+        definition_id_override = "a7bcc9d8-13b3-4e49-b80d-d020b90045e3"
     response: api.CreateDestinationResponse = airbyte_instance.destinations.create_destination(
         models.DestinationCreateRequest(
+            definition_id=definition_id_override,
             name=name,
             workspace_id=workspace_id,
             configuration=config,  # Speakeasy API wants a dataclass, not a dict
@@ -795,9 +861,10 @@ def _make_config_api_request(
         "Authorization": f"Bearer {bearer_token}",
         "User-Agent": "PyAirbyte Client",
     }
+    full_url = config_api_root + path
     response = requests.request(
         method="POST",
-        url=config_api_root + path,
+        url=full_url,
         headers=headers,
         json=json,
     )
@@ -805,8 +872,16 @@ def _make_config_api_request(
         try:
             response.raise_for_status()
         except requests.HTTPError as ex:
+            error_message = f"API request failed with status {response.status_code}"
+            if response.status_code == HTTPStatus.FORBIDDEN:  # 403 error
+                error_message += f" (Forbidden) when accessing: {full_url}"
             raise AirbyteError(
+                message=error_message,
                 context={
+                    "full_url": full_url,
+                    "config_api_root": config_api_root,
+                    "path": path,
+                    "status_code": response.status_code,
                     "url": response.request.url,
                     "body": response.request.body,
                     "response": response.__dict__,
