@@ -64,8 +64,35 @@ class CloudConnection:
             client_secret=self.workspace.client_secret,
         )
 
+    @classmethod
+    def _from_connection_response(
+        cls,
+        workspace: CloudWorkspace,
+        connection_response: ConnectionResponse,
+    ) -> CloudConnection:
+        """Create a CloudConnection from a ConnectionResponse."""
+        result = cls(
+            workspace=workspace,
+            connection_id=connection_response.connection_id,
+            source=connection_response.source_id,
+            destination=connection_response.destination_id,
+        )
+        result._connection_info = connection_response  # noqa: SLF001 # Accessing Non-Public API
+        return result
+
     # Properties
 
+    @property
+    def name(self) -> str | None:
+        """Get the display name of the connection, if available.
+
+        E.g. "My Postgres to Snowflake", not the connection ID.
+        """
+        if not self._connection_info:
+            self._connection_info = self._fetch_connection_info()
+
+        return self._connection_info.name
+
     @property
     def source_id(self) -> str:
         """The ID of the source."""
@@ -130,13 +157,13 @@ class CloudConnection:
 
     @property
     def connection_url(self) -> str | None:
-        """The URL to the connection."""
+        """The web URL to the connection."""
         return f"{self.workspace.workspace_url}/connections/{self.connection_id}"
 
     @property
     def job_history_url(self) -> str | None:
         """The URL to the job history for the connection."""
-        return f"{self.connection_url}/job-history"
+        return f"{self.connection_url}/timeline"
 
     # Run Sync
 
@@ -169,6 +196,13 @@ class CloudConnection:
 
         return sync_result
 
+    def __repr__(self) -> str:
+        """String representation of the connection."""
+        return (
+            f"CloudConnection(connection_id={self.connection_id}, source_id={self.source_id}, "
+            f"destination_id={self.destination_id}, connection_url={self.connection_url})"
+        )
+
     # Logs
 
     def get_previous_sync_logs(
