@@ -40,6 +40,7 @@ from typing import TYPE_CHECKING, Any
 
 from airbyte import exceptions as exc
 from airbyte._util import api_util, text_util
+from airbyte._util.api_util import get_web_url_root
 from airbyte.cloud.connections import CloudConnection
 from airbyte.cloud.connectors import CloudDestination, CloudSource
 from airbyte.destinations.base import Destination
@@ -72,8 +73,8 @@ class CloudWorkspace:
 
     @property
     def workspace_url(self) -> str | None:
-        """The URL of the workspace."""
-        return f"{self.api_root}/workspaces/{self.workspace_id}"
+        """The web URL of the workspace."""
+        return f"{get_web_url_root(self.api_root)}/workspaces/{self.workspace_id}"
 
     # Test connection and creds
 
@@ -157,7 +158,7 @@ class CloudWorkspace:
                 are not allowed. Defaults to `True`.
             random_name_suffix: Whether to append a random suffix to the name.
         """
-        source_config_dict = source.get_config().copy()
+        source_config_dict = source._hydrated_config.copy()  # noqa: SLF001 (non-public API)
         source_config_dict["sourceType"] = source.name.replace("source-", "")
 
         if random_name_suffix:
@@ -205,7 +206,7 @@ class CloudWorkspace:
             random_name_suffix: Whether to append a random suffix to the name.
         """
         if isinstance(destination, Destination):
-            destination_conf_dict = destination.get_config().copy()
+            destination_conf_dict = destination._hydrated_config.copy()  # noqa: SLF001 (non-public API)
             destination_conf_dict["destinationType"] = destination.name.replace("destination-", "")
             # raise ValueError(destination_conf_dict)
         else:
@@ -375,7 +376,10 @@ class CloudWorkspace:
         *,
         name_filter: Callable | None = None,
     ) -> list[CloudConnection]:
-        """List connections by name in the workspace."""
+        """List connections by name in the workspace.
+
+        TODO: Add pagination support
+        """
         connections = api_util.list_connections(
             api_root=self.api_root,
             workspace_id=self.workspace_id,
@@ -385,11 +389,9 @@ class CloudWorkspace:
             client_secret=self.client_secret,
         )
         return [
-            CloudConnection(
+            CloudConnection._from_connection_response(  # noqa: SLF001 (non-public API)
                 workspace=self,
-                connection_id=connection.connection_id,
-                source=None,
-                destination=None,
+                connection_response=connection,
             )
             for connection in connections
             if name is None or connection.name == name
@@ -401,7 +403,10 @@ class CloudWorkspace:
         *,
         name_filter: Callable | None = None,
     ) -> list[CloudSource]:
-        """List all sources in the workspace."""
+        """List all sources in the workspace.
+
+        TODO: Add pagination support
+        """
         sources = api_util.list_sources(
             api_root=self.api_root,
             workspace_id=self.workspace_id,
@@ -411,9 +416,9 @@ class CloudWorkspace:
             client_secret=self.client_secret,
         )
         return [
-            CloudSource(
+            CloudSource._from_source_response(  # noqa: SLF001 (non-public API)
                 workspace=self,
-                connector_id=source.source_id,
+                source_response=source,
             )
             for source in sources
             if name is None or source.name == name
@@ -425,7 +430,10 @@ class CloudWorkspace:
         *,
         name_filter: Callable | None = None,
     ) -> list[CloudDestination]:
-        """List all destinations in the workspace."""
+        """List all destinations in the workspace.
+
+        TODO: Add pagination support
+        """
         destinations = api_util.list_destinations(
             api_root=self.api_root,
             workspace_id=self.workspace_id,
@@ -435,9 +443,9 @@ class CloudWorkspace:
             client_secret=self.client_secret,
         )
         return [
-            CloudDestination(
+            CloudDestination._from_destination_response(  # noqa: SLF001 (non-public API)
                 workspace=self,
-                connector_id=destination.destination_id,
+                destination_response=destination,
             )
             for destination in destinations
             if name is None or destination.name == name
