@@ -51,6 +51,7 @@ def get_source(  # noqa: PLR0913 # Too many arguments
     config_change_callback: ConfigChangeCallback | None = None,
     streams: str | list[str] | None = None,
     version: str | None = None,
+    use_python: bool | Path | str | None = None,
     pip_url: str | None = None,
     local_executable: Path | str | None = None,
     docker_image: bool | str | None = None,
@@ -82,6 +83,13 @@ def get_source(  # noqa: PLR0913 # Too many arguments
         version: connector version - if not provided, the currently installed version will be used.
             If no version is installed, the latest available version will be used. The version can
             also be set to "latest" to force the use of the latest available version.
+        use_python: (Optional.) Python interpreter specification:
+            - True: Use current Python interpreter. (Inferred if `pip_url` is set.)
+            - False: Use Docker instead.
+            - Path: Use interpreter at this path.
+            - str: Use specific Python version. E.g. "3.11" or "3.11.10". If the version is not yet
+                installed, it will be installed by uv. (This generally adds less than 3 seconds
+                to install times.)
         pip_url: connector pip URL - if not provided, the pip url will be inferred from the
             connector name.
         local_executable: If set, the connector will be assumed to already be installed and will be
@@ -112,6 +120,7 @@ def get_source(  # noqa: PLR0913 # Too many arguments
         executor=get_connector_executor(
             name=name,
             version=version,
+            use_python=use_python,
             pip_url=pip_url,
             local_executable=local_executable,
             docker_image=docker_image,
