@@ -18,8 +18,8 @@ CONVERSION_MAP = {
     "number": sqlalchemy.types.DECIMAL(38, 9),
     "boolean": sqlalchemy.types.BOOLEAN,
     "date": sqlalchemy.types.DATE,
-    "timestamp_with_timezone": sqlalchemy.types.TIMESTAMP,
-    "timestamp_without_timezone": sqlalchemy.types.TIMESTAMP,
+    "timestamp_with_timezone": sqlalchemy.types.TIMESTAMP(timezone=True),
+    "timestamp_without_timezone": sqlalchemy.types.TIMESTAMP(timezone=False),
     "time_with_timezone": sqlalchemy.types.TIME,
     "time_without_timezone": sqlalchemy.types.TIME,
     # Technically 'object' and 'array' as JSON Schema types, not airbyte types.
@@ -142,7 +142,7 @@ class SQLTypeConverter:
                 # This is a class. Call its constructor.
                 sql_type = sql_type()
 
-            return sql_type
+            return sql_type  # type: ignore[return-value]
 
         json_schema_type = json_schema_property_def.get("type", None)
         json_schema_format = json_schema_property_def.get("format", None)
@@ -151,7 +151,7 @@ class SQLTypeConverter:
             return sqlalchemy.types.DATE()
 
         if json_schema_type == "string" and json_schema_format == "date-time":
-            return sqlalchemy.types.TIMESTAMP()
+            return sqlalchemy.types.TIMESTAMP(timezone=True)
 
         if json_schema_type == "array":
             return sqlalchemy.types.JSON()
